<?php
	interface nodo {
		function predice($instancia);
	}
	
	class nodo_terminal implements nodo {
		public $valor;
		
		function __construct($valor)
		{
			$this->valor = $valor;
		}
		
		function predice($instancia)
		{
			return $this->valor;
		}
	}
	
	class nodo_no_terminal implements nodo {
		public $variable;
		public $ramas;
		
		function __construct($variable, $ramas)
		{
			$this->variable = $variable;
			$this->ramas = $ramas;
		}
		
		function predice($instancia)
		{
			$valor = $instancia[$this->variable];
			
			if (isset($this->ramas[$valor])) {
				return $this->ramas[$valor]->predice($instancia);
			}
		}
	}

	function calcula_bias($instancias, $variable)
	{
		$cuentas = array_count_values(array_column($instancias, $variable));
		$max = max($cuentas);
		
		return [ 
			'valor' => array_search($max, $cuentas),
			'porcentaje' => $max / count($instancias)
		];
	}
	
	function ramifica($instancias, $variable)
	{
		$ramas = [ ];
		
		foreach ($instancias as $instancia) {
			$ramas[$instancia[$variable]][] = $instancia;
		}
		
		return $ramas;
	}
	
	function construye_arbol($instancias, $variables)
	{
		if (empty($instancias)) {
			return null;
		}
		
		$bias = calcula_bias($instancias, 'clase');
		
		if (empty($variables) || $bias['porcentaje'] > 0.85) {
			return new nodo_terminal($bias['valor']);
		}
		
		$mejor_bias = [ 'valor' => null, 'porcentaje' => 0];
		$mejor_variable = null;
		
		foreach ($variables as $variable) {
			$probar_bias = calcula_bias($instancias, $variable);
			
			if ($probar_bias['porcentaje'] > $mejor_bias['porcentaje']) {
				$mejor_bias = $probar_bias;
				$mejor_variable = $variable;
			}
		}
		
		$previo = ramifica($instancias, $mejor_variable);
		unset($variables[array_search($mejor_variable, $variables)]);
		$ramas = [ ];
		
		foreach ($previo as $valor => $grupo) {
			$ramas[$valor] = construye_arbol($grupo, $variables);
		}
		
		return new nodo_no_terminal($mejor_variable, $ramas);
	}
	
	// script principal

	$filas = file('_jugar.csv', FILE_IGNORE_NEW_LINES|FILE_SKIP_EMPTY_LINES);
	$variables = str_getcsv($filas[0]);
	$instancias = [ ];
	
	for ($i = 1; $i < count($filas); ++$i) {
		$instancias[] = array_combine($variables, str_getcsv($filas[$i]));
	}
	
	unset($variables[array_search('clase', $variables)]);
	$arbol = construye_arbol($instancias, $variables);
	
	var_dump($arbol);
?>